<?php

/**
 * Callback for: `connectid/register`
 */
function _connectid_login__register() {
  if (!connectid_login_is_auth_enabled()) {
    // TODO: Logging
    drupal_set_message(t("Sorry but login via ConnectID is currently unavailable. Please try in a few minutes."), 'error');
    drupal_goto();
  }

  $drupalOathLogin = url('connectid/login', [
    'absolute' => TRUE,
    'query'    => ['destination' => $_GET['destination'] ?? '/'],
  ]);

  $registration_params = [
    'returnUrl' => $drupalOathLogin,
    // TODO: Better error landing page
    'errorUrl'  => url($_GET['destination'] ?? '/', ['absolute' => TRUE]),
  ];

  if ($email = filter_input(INPUT_GET, 'email', FILTER_VALIDATE_EMAIL)) {
    $registration_params['credential'] = $email;
  }

  $provider = connectid_get_provider();
  $registrationUrl = $provider->getRegistrationUrl($registration_params);


  // Remove _GET[destination] to prevent issues with drupal_goto
  unset($_GET['destination']);
  // Send the user to the authentication page.
  drupal_goto($registrationUrl, [], 302);
}

/**
 * Callback for: `connectid/login`
 */
function _connectid_login__start() {
  if (!connectid_login_is_auth_enabled()) {
    // TODO: Logging
    drupal_set_message(t("Sorry but login via ConnectID is currently unavailable. Please try in a few minutes."), 'error');
    drupal_goto();
  }

  $provider = connectid_get_provider();
  $authorizationUrl = $provider->getAuthorizationUrl([
    'redirect_uri' => _connectid_login_get_oauth_redirect_uri(),
  ]);

  // Get the state generated for you and store it to the session.
  drupal_session_start();
  $_SESSION[CONNECTID_SESS_STATE] = $provider->getState();

  // Remove _GET[destination] to prevent issues with drupal_goto
  unset($_GET['destination']);

  // Send the user to the authentication page.
  drupal_goto($authorizationUrl, [], 302);
}

/*
 * Callback for: `connectid/login/authorized`
 */
function _connectid_login__authorized() {
  global $user;
  $params = drupal_get_query_parameters();

  // Return 404 if the code parameter is not present
  if (empty($params['code'])) {
    drupal_set_message(t("An error as occurred while logging in. Please try again and if the error persist please contact us."), 'error');
    watchdog('connectid_login', 'Unable to authenticate: Missing oauth code.', [], WATCHDOG_WARNING);

    drupal_goto();
  }
  // Stop the flow if we get a wrong oauth state
  elseif (!_connectid_login_is_valid_state()) {
    unset($_SESSION['oauth2state']);
    drupal_set_message(t("An error as occurred while logging in. Please try again and if the error persist please contact us."), 'error');
    watchdog('connectid_login', 'Unable to authenticate: Invalid oauth state.', [], WATCHDOG_WARNING);

    drupal_goto();
  }

  try {
    $provider = connectid_get_provider();

    // Try to get an access token using the authorization code grant.
    $accessToken = $provider->getAccessToken('authorization_code', [
      'code' => $params['code'],
      'redirect_uri' => _connectid_login_get_oauth_redirect_uri(),
    ]);

    /** @var \ConnectID\Api\DataModel\ConnectIdProfile $resourceOwner */
    $resourceOwner = $provider->getResourceOwner($accessToken);
    _connectid_login_oauth_register_user_login($resourceOwner);

    if (user_is_logged_in()) {
      // Mark that the current session has been opened via ConnectID
      $_SESSION[SESSION_VIA_CONNECTID] = REQUEST_TIME;
      // Permanently store the token associated with the user.
      connectid_login_store_access_token($user, $accessToken);
    }
    else {
      // Something went wrong
      drupal_set_message(t("An error as occurred while logging in. Please try again and if the error persist please contact us."), 'error');
      watchdog('connectid_login',
        'The user was not logged in without any apparent reason. ',
        [],
        WATCHDOG_ERROR
      );
    }
  }
  /** @noinspection PhpRedundantCatchClauseInspection */
  catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $exception) {
    watchdog_exception('connectid_login', $exception);
  }

  // Whatever happens we need to direct the user to the destination page.
  drupal_goto();
}

/*  ==========  Callbacks toolbox  ==========  */

/**
 * Build the "authorized" return usi, used both in the authorization phase and in
 *  the token retrieval.
 *
 * @return string Return URI with an encoded destination query
 */
function _connectid_login_get_oauth_redirect_uri() {
  /*
   * Use drupal standard 'destination' GET parameter for defining the user's
   * post-authorization destination.
   * Fallback to homepage if not provided.
   */
  $destination = $_GET['destination'] ?? '/';

  // Build the final redirect uri.
  return url('connectid/login/authorized', [
    'absolute' => TRUE,
    'query'    => ['destination' => $destination],
  ]);
}

/**
 * Checks if the current OAuth2 state is valid.
 *
 * @return bool
 */
function _connectid_login_is_valid_state() {
  if (isset($_GET['state'])
    && isset($_SESSION[CONNECTID_SESS_STATE])
    && $_GET['state'] === $_SESSION[CONNECTID_SESS_STATE]
  ) {
    return TRUE;
  }

  return FALSE;
}
