<?php

class ConnectidOrderFormGenerationError extends \RuntimeException {};

/**
 * Builds a subscription form for ConnectID.
 *
 * @param array $form
 * @param array $form_state
 * @param array $settings
 *   Settings for the generation fo the form.
 *
 * @see _connectid_order_get_product_form_element()
 * @see _connectid_order_get_payment_method_form_element()
 *
 * @return mixed
 */
function connectid_order_subscribe_form($form, $form_state, $settings = []) {
  // Wrap the form in a try/catch to handle module misconfiguration nicely.
  try {
    $form['product'] = _connectid_order_get_product_form_element($form, $form_state, $settings);

    // Get default form and tweak it
    $form['billing_address'] = _connectid_order_get_address_fieldset();
    $form['billing_address']['#title'] = t('Billing address');
    $form['billing_address']['#description'] = t('Fill the address for invoicing purposes.');

    // Set required fields
    $required_elements = ['street', 'postalPlace', 'phone'];
    foreach ($required_elements as $item_key) {
      $form['billing_address'][$item_key]['#required'] = TRUE;
    }

    $form['billing_address']['email'] = [
      '#type'     => 'textfield',
      '#title'    => t('Email'),
      '#required' => TRUE,
    ];

    $form['separate_shipping_address'] = [
      '#type'  => 'checkbox',
      '#title' => t('Use different shipping address'),
    ];

    $form['shipping_address'] = _connectid_order_get_address_fieldset();
    $form['shipping_address']['#title'] = t('Shipping address');
    $form['shipping_address']['#description'] = t(
      'Fill the address for shipping purposes.'
    );

    // Get the payment method form element.
    $form['paymentMethod'] = _connectid_order_get_payment_method_form_element($form, $form_state, $settings);

    // Allow setting a redirect URL after purchase.
    if (isset($settings['redirect_url'])) {
      $form['redirect_url'] = [
        '#type'  => 'value',
        '#value' => $settings['redirect_url'],
      ];
    }

    $form['actions'] = [
      '#type'   => 'actions',
      '#weight' => 50,
    ];
    $form['actions']['submit'] = [
      '#type'        => 'submit',
      '#value'       => t('Submit order'),
      '#description' => t('Verify and submit your order.'),
    ];
  }
  catch (ConnectidOrderFormGenerationError $e) {
    // Log the error.
    watchdog(
      'connectid_order',
      'Form generation error: @msg',
      ['@msg' => $e->getMessage()],
      WATCHDOG_ERROR
    );

    // Print a nice(r) message for the user.
    $user_error = t(
      'Could not generate the form. If the problem persists please <a href="mailo:@mail">contact an administrator.</a>',
      ['@mail' => variable_get('site_mail')]
    );

    drupal_set_message($user_error, 'error');
    $form['_generation_error'] = [
      '#markup' => $user_error,
    ];
  }

  return $form;
}


/**
 * @param $form
 * @param $form_state
 */
function connectid_order_subscribe_form_validate($form, &$form_state) {
  $input = &$form_state['values'];

  _connectid_order_subscribe_form_validate_address($form['billing_address'], $input['billing_address']);

  if ($input['separate_shipping_address']) {
    _connectid_order_subscribe_form_validate_address($form['shipping_address'], $input['shipping_address']);
  }
}

/**
 * @param $fieldset
 * @param $fieldset_values
 */
function _connectid_order_subscribe_form_validate_address(&$fieldset, $fieldset_values) {
  switch ($fieldset_values['recipient_type']) {
    case 'company':
      if (empty($fieldset_values['companyName'])) {
        form_error($fieldset['companyName'], t("Company name is required when selecting 'Company' as 'Recipient type'"));
      }
      break;

    case 'private':
    default:
      if (empty($fieldset_values['firstName'])) {
        form_error($fieldset['firstName'], t("First name is required when selecting 'Private' as 'Recipient type'"));
      }
      if (empty($fieldset_values['lastName'])) {
        form_error($fieldset['lastName'], t("Last name is required when selecting 'Private' as 'Recipient type'"));
      }
  }
}

function connectid_order_subscribe_form_submit($form, &$form_state) {
  // Allow setting a destination URL from the form, or else inherit it from the URL.
  $destination = $form_state['values']['redirect_url'] ?? drupal_get_destination()['destination'];
  // Process the order.
  connectid_order_subscribe_process($form_state['values'], $destination);
}

/**
 * Callback for page '/connectid/subscribe/complete'
 *
 * @throws \Exception
 */
function connectid_order_subscribe_complete_page_callback() {
    $form_data = _connectid_order_subscription_form_data_get();
    // If a user is missing not connected via ConnectID should not be allowed to land on this page.
    if (empty($form_data) || !connectid_login_is_connectid_session()) {
        $options = [];
        if (isset($_GET['destination'])) {
            $options['query']['destination'] = $_GET['destination'];
            unset($_GET['destination']);
        }
        drupal_goto('connectid/subscribe', $options);
    }

    $destination = drupal_get_destination();
    connectid_order_subscribe_handle_complete($form_data, $destination['destination']);
}

/**
 * @param array $form_input
 *
 * @throws \Exception
 */
function connectid_order_subscribe_handle_complete(array $form_input, string $destination = '') {
    $order = _connectid_order_build_order_from_input_form($form_input);
    $auth_token = connectid_login_get_access_token();
    $connectid = connectid_get_provider();

    if (!$connectid->submitApiOrder($auth_token, $order)) {
        drupal_set_message(t("We are sorry for the inconvenience, please wait a minute and try again. If the problem persists please contact the support."));
    }
    else {
        connectid_order_store_order($order);
        // FIXME: Should we do it here?
        _connectid_order_subscription_form_data_clear();

        $destination_options = [];
        if ($destination) {
          $parsed_destination = drupal_parse_url($destination);
          $destination_options['query']['destination'] = url($parsed_destination['path'], ['query'=>$parsed_destination['query']]);
          unset($_GET['destination']);
        }

        $returnUrl = url($parsed_destination['path'], ['query'=>$destination_options, 'absolute' => TRUE]);
        $url = $connectid->getCompleteOrderUrl($order, $returnUrl, $returnUrl);
        unset($_GET['destination']);
        drupal_goto($url);
    }
}


/**
 * Generates a Order object based on the user input.
 * Note that little validation is done at this step.
 *
 * @param array $form_input
 *   Array of input values. Should ber validated and structured like connectid_order_subscribe_form()
 *
 * @return bool|\ConnectID\Api\DataModel\Order
 */
function _connectid_order_build_order_from_input_form(array $form_input) {
    $order = new \ConnectID\Api\DataModel\Order();

    $order->withOrderDate(\DateTimeImmutable::createFromFormat('U', REQUEST_TIME))
      ->withPaymentMethod($form_input['paymentMethod'])
      ->withCurrency('NOK');

    $billing_address = _connectid_order_build_address_from_input_form($form_input['billing_address']);
    $order->withPayer($billing_address);

    // TODO
    $all_coupons = variable_get('connectid_order_coupons', []);
    $coupon = $form_input['product'];
    $coupon_data = NULL;
    foreach ($all_coupons as $product_id => $coupon_list) {
        if (isset($coupon_list[$coupon])) {
            $coupon_data = unserialize($coupon_list[$coupon]);
            break;
        }
    }
    if (empty($coupon_data)) {
        return FALSE;
    }
    $coupon = \ConnectID\Api\DataModel\CouponType::create($coupon_data);

    $order->withOrderAmount($coupon->getCouponPrice());

    $order_line_alpha = new \ConnectID\Api\DataModel\OrderLine();

    $order_line_alpha->withQuantity(1);
    $order_line_alpha->withProductSpecType('coupon');

    $order_line_alpha->withProductSpecCode($coupon->getCouponCode());
    $order_line_alpha->withProductSpecNo($coupon->getCouponNumber());
    $order_line_alpha->withUnitPrice($coupon->getCouponPrice());

    if ($form_input['separate_shipping_address'] ?? FALSE) {
        $shipping_address = _connectid_order_build_address_from_input_form($form_input['shipping_address']);
        $order_line_alpha->withReceiver($shipping_address);
    }

    $context = [
      'order' => clone $order,
      'coupon' => clone $coupon,
      'source' => 'build_order_from_input_form',
    ];
    drupal_alter('connectid_order_order_line', $order_line_alpha, $context);
    $order->withOrderLine($order_line_alpha);

    return $order;
}


/**
 * @return array
 *  Default fieldset for ConnectID Address, no fields are marked as required.
 */
function _connectid_order_get_address_fieldset() {
    $fieldset = [
      '#type'        => 'fieldset',
      '#title'       => t('Address'),
      '#description' => t('Fill the address.'),
      '#tree'        => TRUE,
      '#collapsible' => FALSE,
      '#collapsed'   => FALSE,
      '#attached'    => [
        'library' => [
          ['system', 'drupal.collapse'],
        ],
      ],
    ];

    $fieldset['recipient_type'] = [
      '#type'    => 'radios',
      '#title'   => t('Recipient type'),
      '#options' => [
        'company' => t('Company'),
        'private' => t('Private'),
      ],
    ];

    $fieldset['companyName'] = [
      '#type'  => 'textfield',
      '#title' => t('Company'),
    ];

    $fieldset['firstName'] = [
      '#type'  => 'textfield',
      '#title' => t('First name'),
    ];

    $fieldset['lastName'] = [
      '#type'  => 'textfield',
      '#title' => t('Last name'),
    ];

    $fieldset['street'] = [
      '#type'     => 'textfield',
      '#title'    => t('Address'),
    ];

    $fieldset['postalCode'] = [
      '#type'             => 'textfield',
      '#title'            => t('Postal code'),
      '#element_validate' => ['element_validate_integer_positive'],
    ];

    $fieldset['postalPlace'] = [
      '#type'     => 'textfield',
      '#title'    => t('City/Area'),
    ];

    $fieldset['phone'] = [
      '#type'     => 'textfield',
      '#title'    => t('Telephone'),
    ];

    return $fieldset;
}

/**
 * Given a validated input from a form generated by _connectid_order_get_address_fieldset(),
 *  will return a fully populated Address object.
 * No validation is done, so it could generate an object not valid for ConnectID.
 *
 * @param array $form_input
 *
 * @return \ConnectID\Api\DataModel\Address
 */
function _connectid_order_build_address_from_input_form(array $form_input) {
  if (!empty($form_input['email'])) {
    $form_input['emails'] = [
      $form_input['email'],
    ];
    unset($form_input['email']);
  }

  if (!empty($form_input['phone'])) {
    $form_input['phoneNumbers'] = [
      $form_input['phone'],
    ];
    unset($form_input['phone']);
  }

  unset($form_input['recipient_type']);
  $address = \ConnectID\Api\DataModel\Address::create($form_input);

  return $address;
}

/**
 * Generates the "coupon code" form element.
 * Allows a few $settings to be used, see docs below.
 *
 * @param array $form
 * @param array $form_state
 * @param array $settings
 *   Associative array of options:
 *     - default_coupon (string, default NULL): Set a predefined coupon code
 *     - deny_coupon_selection (bool, default FALSE): Does not allow a user to
 *         select the coupon. Note that in this case a coupon code MUST be passed
 *         as $settings['default_coupon'] or as a GET parameter <code>coupon_code</code>.
 *     - deny_coupon_from_url (bool, default: FALSE): Prevents the coupon from
 *         being passed as GET parameter.
 *     - hide_selection_if_provided (bool, default TRUE): Hides the selection of coupons when a value
 *         is provided as URL parameter.
 *
 * @return array
 */
function _connectid_order_get_product_form_element(array $form, array $form_state, array $settings = []) {
  $element = [
    '#title'       => t('Product'),
    '#description' => t('Select which product you are purchasing.'),
    '#required'    => TRUE,
  ];

  // Get default coupon from the settings.
  $default_coupon = $settings['default_coupon'] ?? NULL;
  // Get the coupon passed as parameter
  $url_coupon = filter_input(INPUT_GET, 'coupon_code', FILTER_SANITIZE_STRING);
  $available_coupons = connectid_order_get_active_coupons_as_options();

  /**
   * Check if user can select a different coupon from the default or not.
   * @var bool $deny_coupon_user_selection
   */
  $deny_coupon_user_selection = (bool) (isset($settings['deny_coupon_selection'])) ?  $settings['deny_coupon_selection'] : FALSE;

  /**
   * Check if user can select a different coupon from the default or not.
   * @var bool $deny_coupon_user_selection
   */
  $hide_selection_if_provided = (bool) (isset($settings['hide_selection_if_provided'])) ?  $settings['hide_selection_if_provided'] : TRUE;

  /**
   * Check if a predefined coupon code can be passed as GET parameter or not.
   * @var bool $deny_coupon_from_url
   */
  $deny_coupon_from_url = (bool) (isset($settings['deny_coupon_from_url']) ? $settings['deny_coupon_from_url'] : FALSE);

  // Check if the provided value from the URL is valid.
  $is_url_coupon_valid = !empty($url_coupon) && isset($available_coupons[$url_coupon]);
  if ($deny_coupon_user_selection || ($hide_selection_if_provided && $is_url_coupon_valid)) {
    // Use the coupon code from URL, or fallback to the default value.
    if (!$deny_coupon_from_url && !empty($url_coupon)) {
      $value = $url_coupon;
    }
    else {
      $value = $default_coupon;
    }

    // At this point a value is strictly required.
    if (empty($value)) {
      if (empty($default_coupon) && $deny_coupon_from_url) {
        throw new ConnectidOrderFormGenerationError("Default coupon code is not provided, whilst 'deny coupon selection' and 'deny coupon from URL' option are set.");
      } else {
        throw new ConnectidOrderFormGenerationError("Coupon code is not provided, whilst 'deny coupon selection' option is set.");
      }
    }

    // If the selected coupon is not valid and user coupon selection is disabled we cannot render this form.
    if (!isset($available_coupons[$value])) {
      throw new ConnectidOrderFormGenerationError("Invalid coupon code selected and 'deny coupon selection' option is set.");
    }

    // Hardcoded coupon if user can't select it.
    $element['#type'] = 'value';
    $element['#value'] = $value;
  }
  else {
    // Set the type to be a radio buttons, to allow the user to select a coupon code.
    $element['#type'] = 'radios';
    $element['#options'] = $available_coupons;

    // Highlight the current choice.
    if (!$deny_coupon_from_url && $url_coupon) {
      $element['#default_value'] = $url_coupon;
    }
    elseif ($default_coupon) {
      $element['#default_value'] = $default_coupon;
    }

    // Register a notice if we're trying to set a non-existing default value.
    if (isset($element['#default_value']) && !isset($available_coupons[$element['#default_value']])) {
      watchdog(
        'connectid_order',
        "Trying to pre-select a non existing coupon code: @code",
        ['@code' => $element['#default_value']],
        WATCHDOG_NOTICE
      );
    }
  }

  return $element;
}

/**
 * Builds a "paymentMethod" form element to use in a ConnectID Order form
 *
 * @param array $form
 *   Drupal FormAPI array.
 * @param array $form_state
 *   Drupal standard Form State
 * @param array $settings
 *   Current form's settings.
 *
 * @return array
 *   Form element
 */
function _connectid_order_get_payment_method_form_element(array $form, array $form_state, array $settings) {
  /*
   * Get the allowed payments for the current options,
   * or fallback to the global enabled payment types.
   */
  if (!empty($settings['payment_methods'])) {
    $payment_methods = $settings['payment_methods'];
  }
  else {
    $payment_methods = connectid_order_get_active_payment_types_as_options();
  }

  $element =  [
    '#type'     => 'radios',
    '#title'    => t('Payment method'),
    '#options'  => $payment_methods,
    '#required' => TRUE,
  ];

  // If we only have 1 option as payment gateway set this as a value, as user can't select anything.
  if (count($payment_methods) === 1) {
    $element['#type'] = 'value';
    $element['#value'] = key($payment_methods);
  }
  elseif (count($payment_methods) < 1) {
    throw new ConnectidOrderFormGenerationError("There are no payment methods enabled.");
  }

  return $element;
}
