<?php

/**
 * Alters the ConnectID form adding the ConnectID Order configuration.
 *
 * @param array $form
 */
function _connectid_order_connectid_login_admin_alter(&$form) {

  $form['connectid_order'] = [
    '#type'        => 'fieldset',
    '#title'       => t('ConnectID Order Configuration'),
    '#description' => t("Configuration for the ConnectID Order module."),
    '#tree'        => FALSE,
  ];

  $config_form = &$form['connectid_order'];
  $config_form['connectid_order_products_status'] = [
    '#type'        => 'fieldset',
    '#title'       => t('Enabled Products'),
    '#description' => t("Manage enabled products"),
    '#tree'        => TRUE,
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
    '#attached'    => [
      'library' => [
        ['system', 'drupal.collapse'],
      ],
    ],
  ];

  $products = _connectid_order_get_available_products();
  if ($products === FALSE) {
    $config_form['connectid_order_products_status']['#collapsed'] = FALSE;
    $config_form['connectid_order_products_status']['could_not_fetch'] = [
      '#prefix' => '<p style="text-align: center; font-size: 1.25em">',
      '#markup' => t("The Product List is currently unavailable. <br> Please contact your site administrator before doing any change to this page."),
      '#suffix' => '</p>',
    ];
  }
  else {
    $products_status = variable_get('connectid_order_products_status');
    $config_form['connectid_order_products'] = [
      '#type'  => 'value',
      '#value' => [],
    ];
    /** @var \ConnectID\Api\DataModel\ProductType $product */
    foreach ($products as $product) {
      $product_description = t("Company: @company, code: @code, type: @type.", [
        '@company' => $product->getCompanyCode(),
        '@code'    => $product->getProduct(),
        '@type'    => $product->getProductType(),
      ]);

      $config_form['connectid_order_products_status'][$product->getId()] = [
        '#type'          => 'checkbox',
        '#title'         => $product->getDescription(),
        '#description'   => $product_description,
        '#default_value' => isset($products_status[$product->getId()]) ? $products_status[$product->getId()] : FALSE,
      ];
      // Store also the product information for alter reference.
      $config_form['connectid_order_products']['#value'][$product->getId()] = serialize($product->toArray());

      // Allow filtering.
      $products_status[$product->getId()] = FALSE;
    }

    if (0 !== count($missing_products = array_filter($products_status))) {
      drupal_set_message(t("The following products are enabled, but they are missing in ConnectID: @products ", ['@products' => implode(", ", array_keys($missing_products))]), 'error');
    }
  }

  $enabled_products = array_filter(variable_get('connectid_order_products_status', []));
  $config_form['connectid_order_coupons_status'] = [
    '#type'        => 'fieldset',
    '#title'       => t('Enabled Coupons'),
    '#description' => t("Manage enabled coupons"),
    '#tree'        => TRUE,
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
    '#attached'    => [
      'library' => [
        ['system', 'drupal.collapse'],
      ],
    ],
  ];

  if (empty($enabled_products)) {
    $config_form['connectid_order_coupons_status']['#collapsed'] = FALSE;
    $config_form['connectid_order_coupons_status']['no_products'] = [
      '#prefix' => '<p>',
      '#markup' => t("Once you have enabled at the least one product the relative coupons will be fetched."),
      '#suffix' => '</p>',
    ];
  }
  else {
    $coupon_per_product_list = [];
    $products_info = variable_get('connectid_order_products', []);
    foreach (array_keys($enabled_products) as $product_identifier) {
      if (!isset($products_info[$product_identifier])) {
        drupal_set_message(t('Missing product data for: @product', ['product' => $product_identifier]), 'warning');
        continue;
      }
      $info = unserialize($products_info[$product_identifier]);
      $coupon_list = \ConnectID\Api\DataModel\ProductType::create($info);
      $coupon_per_product_list[$product_identifier] = _connectid_order_get_available_coupons($coupon_list);
    }


    if (!empty($coupon_per_product_list)) {
      $coupons_status = connectid_order_get_active_coupons();
      $config_form['connectid_order_coupons'] = [
        '#type'  => 'value',
        '#value' => [],
      ];
      /**
       * @var string $product_key
       * @var \ConnectID\Api\DataModel\CouponTypeList $coupon_list
       */
      foreach ($coupon_per_product_list as $product_key => $coupon_list) {
        if ($coupon_list && $coupon_list->count()) {
          if (!isset($config_form['connectid_order_coupons_status'][$product_key])) {
            $config_form['connectid_order_coupons_status'][$product_key] = [
              '#type'        => 'fieldset',
              '#title'       => t('Coupons for @product', ['@product' => $product_key]),
              '#description' => t("Manage enabled coupons"),
              '#tree'        => TRUE,
              '#collapsible' => TRUE,
              '#collapsed'   => FALSE,
            ];
          }

          /** @var \ConnectID\Api\DataModel\CouponType $coupon */
          foreach ($coupon_list as $coupon) {
            $config_form['connectid_order_coupons_status'][$product_key][$coupon->getId()] = [
              '#type'          => 'checkbox',
              '#title'         => $coupon->getDescription(),
              '#description' => t("Code: @code, Number: @num, Price: @price", [
                '@code'  => $coupon->getCouponCode(),
                '@num'   => $coupon->getCouponNumber(),
                '@price' => $coupon->getCouponPrice(),
              ]),
              '#default_value' => isset($coupons_status[$product_key][$coupon->getId()]) ? $coupons_status[$product_key][$coupon->getId()] : FALSE,
            ];
            $config_form['connectid_order_coupons']['#value'][$product_key][$coupon->getId()] = serialize($coupon->toArray());
          }
        }
      }
    }
  }

}

/**
 * Gets the AccessToken used by the system to fetch data from ConnectID's
 * Client API
 *
 * @return \League\OAuth2\Client\Token\AccessToken|null
 */
function _connectid_order_get_system_access_token() {
  $client_token = NULL;
  $connectid = connectid_get_provider();
  // Try to fetch the Client Token, required to fetch the Client API.
  if (isset($connectid)) {
    try {
      // Fetch the required Client Token
      $client_token = $connectid->getClientCredentialsAccessToken();
    }
    // TODO proper Exception type!
    catch (Exception $exception) {
      watchdog("connectid_order", "Failed to get Client-Token: @message", ['@message' => $exception->getMessage()], WATCHDOG_ERROR);
      drupal_set_message(t("Failed to communicate with the ConnectID API. This error has been logged."), 'error');
    }
  }

  return $client_token;
}

/**
 * @return false|\ConnectID\Api\DataModel\ProductTypeList
 */
function _connectid_order_get_available_products() {
  $token = _connectid_order_get_system_access_token();
  $connectid = connectid_get_provider();

  if (isset($token, $connectid)) {
    return $connectid->getClientApiProducts($token);
  }

  return FALSE;
}

/**
 * @param \ConnectID\Api\DataModel\ProductType $product_type
 *
 * @return false|\ConnectID\Api\DataModel\CouponTypeList
 */
function _connectid_order_get_available_coupons(\ConnectID\Api\DataModel\ProductType $product_type) {
  $connectid = connectid_get_provider();

  try {
    if (isset($connectid)) {
      return $connectid->getClientApiCoupons($product_type);
    }
  }
  /** @noinspection PhpRedundantCatchClauseInspection */
  catch(\Ramsalt\OAuth2\Client\Provider\Exception\InvalidApiResponseException $exception) {
    watchdog_exception('connectid_order',
      $exception,
      'Invalid response: @msg',
      ['@msg' => $exception->getMessage()]
    );
  }

  return FALSE;
}
